/***
 * Simple testprogram for the rtc driver
 * Copyright (c) 2003, Inventronik
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the Inventronik nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: rtc-test.c,v 1.1 2005/01/27 18:14:09 jensi Exp $ *
 */

#if defined(__PCM__)
#include <16f876.h>
#fuses XT, NOWDT, NOPROTECT, PUT, NOBROWNOUT, NOLVP
#elif defined(__PCH__)
#include <18f252.h>
#fuses EC, NOWDT, NOPROTECT, PUT, NOBROWNOUT, NOLVP
#endif

#use delay(clock=6000000)

#zero_ram // erase all used ram cells at initialisation.
#use rs232(baud=19200, xmit=PIN_C6, rcv=PIN_C7, ERRORS)
#include "rtc4513.c"

#define BUFFER_SIZE 32
byte rcvbuf[BUFFER_SIZE];
byte inPtr = 0, outPtr = 0;

#int_rda
void serial_isr() 
{
    byte save;

    rcvbuf[inPtr]=getc();
    save=inPtr;
    inPtr=(inPtr+1) % BUFFER_SIZE;
    if(inPtr==outPtr) {
        inPtr=save;
    }
    output_high(PIN_B0);
}

#define bkbhit (inPtr!=outPtr)
byte bgetc(void)
{
    byte c;

    while(!bkbhit);
    c=rcvbuf[outPtr];
    outPtr=(outPtr+1) % BUFFER_SIZE;
    return c;
}

void main(void)
{
    byte s, m, h, d;
    unsigned long y;

    enable_interrupts(global);
    enable_interrupts(int_rda);

    printf("\r\n\Start...\r\n");
    rtc_init();
    printf("\r\n\Running...\r\n");

    settime(23,58,17); // set time 23:58:17
    setdate(15,11,3); // set date 15.8.2003
    printall();
//  delay_ms(10000);


    // The program will delay for 10 seconds and then display
    // any data that came in during the 10 second delay
    do {
        s = gettime('s', 1);
        m = gettime('m', 0);
        h = gettime('h', 0);
        printf("\r\n-->time: %02d:%02d:%02d", h, m, s);
        d = getdate('d', 1);
        m = getdate('m', 0);
        y = getdate('y', 0);
        printf("\r\n-->date: %02d.%02d.%ld", d, m, y+2000);
        printall();
//    setdow(4); 
        delay_ms(10000);
        printf("\r\nBuffered data => ");
        while(bkbhit) {
            putc(bgetc());
        }
    } while(TRUE);
}
