/***
 * Test program to recive and transmit single characters over RS232/USB
 *     the program does not detect buffer overflows
 *
 * Copyright (c) 2003, Inventronik
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the Inventronik nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: usb-rs232.c,v 1.1 2005/01/27 18:14:09 jensi Exp $ *
 *
 * new 2. Feb 04 - added mem protection for the bootloader
 */

#if defined(__PCM__)
#include <16f876.h>
#fuses XT, NOWDT, NOPROTECT, PUT, NOBROWNOUT, NOLVP
#elif defined(__PCH__)
#include <18f252.h>
#fuses EC, NOWDT, NOPROTECT, PUT, NOBROWNOUT, NOLVP
#endif

#zero_ram // erase all used ram cells at initialisation.

#use delay(clock=6000000)
#use rs232(baud=19200, xmit=PIN_C6, rcv=PIN_C7, ERRORS)

/* reserve space for the bootstrap loader */
#ORG 0x1F00,0x1FFF //for the 8k 16F876/7
void loader() { }

#define BUFFER_SIZE 32
byte rcvbuf[BUFFER_SIZE];
byte inPtr = 0, outPtr = 0;

#int_rda
void serial_isr() 
{
    byte c;

    rcvbuf[inPtr] = c = getc();
    inPtr = (inPtr+1) % BUFFER_SIZE;
    if(inPtr==outPtr) {
        inPtr = outPtr = 0; // buffer full - reset inPtr, outPtr
    } 
}

#define bkbhit (inPtr!=outPtr)
byte bgetc(void)
{
    byte c;

    while(!bkbhit);
    c=rcvbuf[outPtr];
    outPtr=(outPtr+1) % BUFFER_SIZE;
    return c;
}


void main(void)
{
    enable_interrupts(global);
    enable_interrupts(int_rda);

    delay_ms(500);
    printf("\r\nRunning...\r\n");

    // The program will delay for 10 seconds and then display
    // any data that came in during the 10 second delay
    do {
        delay_ms(10000);
        printf("\r\nBuffered data => ");
        while(bkbhit) {
            putc(bgetc());
        }
    } while(1); //endless
}
